/* ----------------------------------------------------------------------------
 * Raima Database Manager
 *
 * Copyright (c) 2012 Raima Inc.,  All rights reserved.
 *
 * Use of this software, whether in source code format, or in executable,
 * binary object code form, is governed by the Raima LICENSE which
 * is fully described in the LICENSE.TXT file, included within this
 * distribution of files.
 * ----------------------------------------------------------------------------
 */

#include <stdio.h>

/* rdmsql.h is required in all files using RDMSQL */
#include "rdmsql.h"

/* EXAMPLE - SQL02 for C using RDMSQL */
/* When you run this example, a new database is created with two tables
 * where one references the other.  Then a single row is created in the
 * referenced table and four rows are created in the referencing table.
 * Finally all the data is read from the database using a join.
 * 
 * The following RDM functions are illustrated:
 *      rsqlAllocConn/rsqlFreeConn
 *      rsqlAllocStmt/rsqlFreeStmt
 *      rsqlOpenDB/rsqlCloseDB
 *      rsqlExecDirect
 *      rsqlFetch
 *
 * They are fully documented in http://docs.raima.com/rdm/.
 * 
 * For simplicity this example does not catch any thrown execptions
 * but good programming practices would dictate that this occur in
 * a full application.
 */

static void create_database(void)
{
    RSQL_ERRCODE rsqlCode;
    HCONN        hDbc;
    HSTMT        hStmt;
    int16_t      ii;

    const char *cmds[] = {
        "CREATE DATABASE SQL02_db",
        "CREATE TABLE PriTab (pid INTEGER PRIMARY KEY, pstr CHAR(30))",
        "CREATE TABLE ForTab (fstr CHAR(30), pid INTEGER REFERENCES PriTab(pid))",
        "COMMIT"
    };

    (void) rsqlAllocConn(&hDbc);
    (void) rsqlAllocStmt(hDbc, &hStmt);

    /* Attempt to drop the database.  If the database does not already exist
     * this will fail and we simply ignore the failure for this sample. */
    (void) rsqlExecDirect(hStmt, "DROP DATABASE SQL02_db");
    (void) rsqlExecDirect(hStmt, "COMMIT");

    for (ii = 0; ii < sizeof(cmds) / sizeof(cmds[0]); ii++)
    {
        rsqlCode = rsqlExecDirect(hStmt, cmds[ii]);
        if (rsqlCode != errSUCCESS)
            break;
    }

    if (rsqlCode != errSUCCESS)
        fprintf(stderr, "Error creating the SQL02 database\n");

    /* Close the newly created database */
    (void) rsqlCloseDB(hDbc, "SQL02_db");

    (void) rsqlFreeStmt(hStmt);
    (void) rsqlFreeConn(hDbc);
}

/* =======================================================================
*/
int32_t main(void)
{
    RSQL_ERRCODE      rsqlCode;
    HCONN             hDbc;
    HSTMT             hStmt;
    const RSQL_VALUE *result;
    uint16_t          numCols;
    int16_t           ii;

    create_database();

    /* Allocate handles */
    (void) rsqlAllocConn(&hDbc);
    (void) rsqlAllocStmt(hDbc, &hStmt);

    /* Open the database in exclusive mode */
    (void) rsqlOpenDB(hDbc, "SQL02_db", "x");

    /* Insert a row into the referenced table */
    (void) rsqlExecDirect(hStmt, "INSERT INTO PriTab VALUES 1, 'referenced 1'");

    /* Insert four rows into the referencing table */
    for (ii = 1; ii < 5; ii++)
    {
        char commandText[80];

        sprintf(commandText, "INSERT INTO ForTab VALUES 'referencing %hu', 1", ii);
        (void) rsqlExecDirect(hStmt, commandText);
    }

    /* Scan through all the existing rows using a join and print out the
     * contents */
    (void) rsqlExecDirect(hStmt, "SELECT pstr, fstr FROM ForTab, PriTab where ForTab.pid = PriTab.pid");

    while ((rsqlCode = rsqlFetch(hStmt, &result, &numCols)) == errSUCCESS)
        printf("%s -- %s\n", result[0].vt.cv, result[1].vt.cv);

    /* Close the database */
    (void) rsqlCloseDB(hDbc, "SQL02_db");

    /* Free handles */
    (void) rsqlFreeStmt(hStmt);
    (void) rsqlFreeConn(hDbc);

    printf("\nPress <Enter> to continue . . .\n");
    getchar();

    return EXIT_SUCCESS;
}

